#include <stdlib.h>
#include <xdc/runtime/System.h>
#include <xdc/std.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>

/* Application headers */
#include "Board.h"
#include "menu.h"
#include "RFQueue.h"
#include "smartrf_settings/smartrf_settings.h"


/***** Defines *****/
/* TX Configuration */
#define DATA_ENTRY_HEADER_SIZE 8   /* Constant header size of a Generic Data Entry */
#define MAX_LENGTH             1000//250//125 /* Max length byte the radio will accept */
#define NUM_DATA_ENTRIES       2   /* NOTE: Only two data entries supported at the moment */
#define NUM_APPENDED_BYTES     2   /* RF_cmdRxHS.rxConf.bIncludeLen = 1: Include the received length

/***** Prototypes *****/
static void rx_callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e);

/***** Variable declarations *****/
static uint8_t packetReceived = false;
static uint16_t* crcOk;
static uint16_t* crcNOk;
static uint8_t* RxIg;
static uint8_t* RxStop;
static uint8_t* RxBufFull;

static int8_t* rssi;

static RF_Object rfObject;
static RF_Handle rfHandle;
static RF_CmdHandle rxCmdHndl = NULL; /* Handle needed to abot the RX command */

#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN (rxDataEntryBuffer, 4);
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment = 4
#else
#error This compiler is not supported.
#endif
static uint8_t rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                                 MAX_LENGTH,
                                                                 NUM_APPENDED_BYTES)];

static dataQueue_t dataQueue;                    /* Receive queue for the RF Code to fill in data */
static rfc_dataEntryGeneral_t* currentDataEntry; /* General data entry structure (type = 0) */
rfc_propRxOutput_t rxStatistics_prop;            /* Output structure for CMD_PROP_RX */

TestCaseResult rx_run()
{
  RF_Params rfParams;
  RF_Params_init(&rfParams);
  
  if( RFQueue_defineQueue(&dataQueue,
                          rxDataEntryBuffer,
                          sizeof(rxDataEntryBuffer),
                          NUM_DATA_ENTRIES,
                          MAX_LENGTH + NUM_APPENDED_BYTES))
  {
    /* Failed to allocate space for all data entries */
    while(1);
  }
  
  RF_cmdPropRxAdv.pOutput = (uint8_t*)&rxStatistics_prop;
  RF_cmdPropRxAdv.pQueue = &dataQueue;           /* Set the Data Entity queue for received data */
  RF_cmdPropRxAdv.rxConf.bAutoFlushIgnored = 1;  /* Discard ignored packets from Rx queue */
  RF_cmdPropRxAdv.rxConf.bAutoFlushCrcErr = 1;   /* Discard packets with CRC error from Rx queue */
  RF_cmdPropRxAdv.maxPktLen = MAX_LENGTH;        /* Implement packet length filtering to avoid PROP_ERROR_RXBUF */
  RF_cmdPropRxAdv.pktConf.bRepeatOk = 1;
  RF_cmdPropRxAdv.pktConf.bRepeatNok = 1;
  
  switch (menu_config.testCase)
  {
  case FskTest:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_fsk.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_fsk.loDivider = 0x0a;
    }  
    RF_cmdPropRadioDivSetup_fsk.centerFreq = ppFrequencies[menu_config.frequency].frequency;
    rfHandle = RF_open(&rfObject, &RF_prop_fsk, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_fsk, &rfParams);    
    break;
    
  case OneMTest:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_1M.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_1M.loDivider = 0x0a;
    }
    RF_cmdPropRadioDivSetup_1M.centerFreq = ppFrequencies[menu_config.frequency].frequency;  
    rfHandle = RF_open(&rfObject, &RF_prop_1M, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_1M, &rfParams);
    break;
    
  case Test500K:
    if(ppFrequencies[menu_config.frequency].frequency == 0x01b1)
    {
      RF_cmdPropRadioDivSetup_500K.txPower = 0x003f;
      RF_cmdPropRadioDivSetup_500K.loDivider = 0x0a;
    }
    RF_cmdPropRadioDivSetup_500K.centerFreq = ppFrequencies[menu_config.frequency].frequency;  
    rfHandle = RF_open(&rfObject, &RF_prop_500K, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup_500K, &rfParams);
    break;
    
  default:
    
    break;
  }
  
  /* Set the frequency */
  RF_cmdFs.frequency = ppFrequencies[menu_config.frequency].frequency;
  RF_cmdFs.fractFreq = ppFrequencies[menu_config.frequency].fractFreq;
  
  RF_runCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);
  rxCmdHndl = RF_postCmd(rfHandle, (RF_Op*)&RF_cmdPropRxAdv, RF_PriorityNormal, &rx_callback, RF_EventRxEntryDone);
  
  crcOk     = &rxStatistics_prop.nRxOk;
//  crcNOk    = &rxStatistics_prop.nRxNok;
//  RxIg      = &rxStatistics_prop.nRxIgnored;
//  RxStop    = &rxStatistics_prop.nRxStopped;
//  RxBufFull = &rxStatistics_prop.nRxBufFull;

  rssi = &rxStatistics_prop.lastRssi;
  
  *crcOk = 0;//*crcNOk = *RxIg = *RxStop = *RxBufFull = 0;
  *rssi = 0;
  while(true)
  {
    if(packetReceived)
    {
      packetReceived = false;
      menu_update_rxScreen(*crcOk, *rssi);
//      menu_update_rxScreen(*crcOk, *crcNOk, *RxIg, *RxStop, *RxBufFull, *rssi);
    }
    
    if (Event_pend(menu_event, Event_Id_NONE, KeyPressEvent, BIOS_NO_WAIT) & KeyPressEvent)
    {
      RF_cancelCmd(rfHandle, rxCmdHndl, 0);
      RF_EventMask result = RF_pendCmd(rfHandle, rxCmdHndl, RF_EventRxEntryDone);
      RF_close(rfHandle);
      return TestAborted;
    }
  }
}

void rx_callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e)
{
  if (e & RF_EventRxEntryDone)
  {
    currentDataEntry = RFQueue_getDataEntry();
    packetReceived = true;
    RFQueue_nextEntry();
  }
}